#!/usr/bin/env python3
"""
Windows Clipboard Manager (System Management Tool)

Parameters:
    --get: Get current clipboard content
    --set TEXT: Set clipboard content
    --monitor: Monitor clipboard changes
    --clear: Clear clipboard content
    --history [N]: Show clipboard history (last N items)
    --save FILE: Save clipboard content to file
    --format FORMAT: Output format (table/json)

Requires: win32clipboard, pillow, tabulate packages
Warning: Some clipboard content types may not be supported
"""

import os
import sys
import json
import time
import base64
import logging
import tempfile
from typing import List, Dict, Optional, Union
from datetime import datetime
from pathlib import Path

import win32clipboard
import win32con
from PIL import ImageGrab, Image
from tabulate import tabulate

# Configure logging
logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s - %(levelname)s - %(message)s'
)
logger = logging.getLogger(__name__)

class ClipboardManager:
    """Manages Windows clipboard operations and history."""
    
    def __init__(self, history_size: int = 100):
        """
        Initialize the ClipboardManager.
        
        Args:
            history_size: Maximum number of items to keep in history
        """
        self.history_size = history_size
        self.history: List[Dict] = []
        self.temp_dir = Path(tempfile.gettempdir()) / "clipboard_manager"
        self.temp_dir.mkdir(exist_ok=True)
        
    def get_clipboard_content(self) -> Dict:
        """
        Get current clipboard content.
        
        Returns:
            Dict: Clipboard content with metadata
        """
        try:
            win32clipboard.OpenClipboard()
            
            formats = []
            format_id = 0
            
            # Get all available formats
            while True:
                try:
                    format_id = win32clipboard.EnumClipboardFormats(format_id)
                    if not format_id:
                        break
                    formats.append(format_id)
                except:
                    break
            
            content = {
                "timestamp": datetime.now().isoformat(),
                "formats": formats,
                "data": None,
                "type": None,
                "size": 0
            }
            
            # Try to get text content
            if win32con.CF_TEXT in formats:
                try:
                    text = win32clipboard.GetClipboardData(win32con.CF_TEXT)
                    content["data"] = text.decode('utf-8')
                    content["type"] = "text"
                    content["size"] = len(content["data"])
                except Exception as e:
                    logger.error(f"Error getting text data: {e}")
            
            # Try to get unicode text content
            elif win32con.CF_UNICODETEXT in formats:
                try:
                    text = win32clipboard.GetClipboardData(win32con.CF_UNICODETEXT)
                    content["data"] = text
                    content["type"] = "text"
                    content["size"] = len(content["data"])
                except Exception as e:
                    logger.error(f"Error getting unicode text data: {e}")
            
            # Try to get image content
            elif win32con.CF_DIB in formats:
                try:
                    image = ImageGrab.grabclipboard()
                    if image:
                        temp_path = self.temp_dir / f"clip_{int(time.time())}.png"
                        image.save(temp_path)
                        content["data"] = str(temp_path)
                        content["type"] = "image"
                        content["size"] = os.path.getsize(temp_path)
                except Exception as e:
                    logger.error(f"Error getting image data: {e}")
            
            # Try to get file list content
            elif win32con.CF_HDROP in formats:
                try:
                    files = win32clipboard.GetClipboardData(win32con.CF_HDROP)
                    content["data"] = files
                    content["type"] = "files"
                    content["size"] = len(files)
                except Exception as e:
                    logger.error(f"Error getting file list data: {e}")
            
            win32clipboard.CloseClipboard()
            return content
            
        except Exception as e:
            logger.error(f"Error accessing clipboard: {e}")
            return {
                "timestamp": datetime.now().isoformat(),
                "error": str(e)
            }
    
    def set_clipboard_content(self, content: Dict) -> bool:
        """
        Set clipboard content from saved data.
        
        Args:
            content: Clipboard content dictionary
            
        Returns:
            bool: True if successful, False otherwise
        """
        try:
            win32clipboard.OpenClipboard()
            win32clipboard.EmptyClipboard()
            
            if content["type"] == "text":
                win32clipboard.SetClipboardText(content["data"])
                
            elif content["type"] == "image":
                image_path = Path(content["data"])
                if image_path.exists():
                    image = Image.open(image_path)
                    output = tempfile.NamedTemporaryFile(delete=False)
                    image.save(output, "BMP")
                    data = output.read()[14:]  # Skip BMP header
                    output.close()
                    os.unlink(output.name)
                    win32clipboard.SetClipboardData(win32con.CF_DIB, data)
                    
            elif content["type"] == "files":
                win32clipboard.SetClipboardData(win32con.CF_HDROP, content["data"])
                
            win32clipboard.CloseClipboard()
            return True
            
        except Exception as e:
            logger.error(f"Error setting clipboard: {e}")
            return False
    
    def monitor_clipboard(self, callback=None) -> None:
        """
        Monitor clipboard for changes.
        
        Args:
            callback: Optional callback function for clipboard changes
        """
        logger.info("Starting clipboard monitoring")
        last_content = None
        
        try:
            while True:
                current_content = self.get_clipboard_content()
                
                if "error" not in current_content:
                    content_hash = hash(str(current_content["data"]))
                    
                    if last_content is None or content_hash != last_content:
                        self.add_to_history(current_content)
                        if callback:
                            callback(current_content)
                        last_content = content_hash
                        
                        logger.info(
                            f"New clipboard content: {current_content['type']} "
                            f"({current_content['size']} bytes)"
                        )
                
                time.sleep(1)
                
        except KeyboardInterrupt:
            logger.info("Clipboard monitoring stopped")
    
    def add_to_history(self, content: Dict) -> None:
        """
        Add content to clipboard history.
        
        Args:
            content: Clipboard content to add
        """
        self.history.append(content)
        if len(self.history) > self.history_size:
            # Remove old temporary files
            old_content = self.history.pop(0)
            if old_content["type"] == "image":
                try:
                    os.remove(old_content["data"])
                except:
                    pass
    
    def search_history(self, query: str) -> List[Dict]:
        """
        Search clipboard history.
        
        Args:
            query: Search query string
            
        Returns:
            List[Dict]: Matching clipboard entries
        """
        results = []
        
        for content in self.history:
            if content["type"] == "text":
                if query.lower() in str(content["data"]).lower():
                    results.append(content)
            elif content["type"] == "files":
                if any(query.lower() in str(f).lower() for f in content["data"]):
                    results.append(content)
        
        return results
    
    def clear_history(self) -> None:
        """Clear clipboard history and temporary files."""
        for content in self.history:
            if content["type"] == "image":
                try:
                    os.remove(content["data"])
                except:
                    pass
        self.history.clear()
    
    def __del__(self):
        """Cleanup temporary files on object destruction."""
        self.clear_history()
        try:
            os.rmdir(self.temp_dir)
        except:
            pass

def format_clipboard_data(content: Dict, format_type: str = "table") -> str:
    """
    Format clipboard content for display.
    
    Args:
        content: Clipboard content dictionary
        format_type: Output format ("table" or "json")
        
    Returns:
        str: Formatted content
    """
    if format_type == "json":
        return json.dumps(content, indent=2)
    
    if "error" in content:
        return f"Error: {content['error']}"
    
    data = [
        ["Timestamp", content["timestamp"]],
        ["Type", content["type"]],
        ["Size", f"{content['size']} bytes"],
        ["Formats", ", ".join(map(str, content["formats"]))]
    ]
    
    if content["type"] == "text":
        preview = str(content["data"])[:100] + "..." if len(str(content["data"])) > 100 else str(content["data"])
        data.append(["Preview", preview])
    elif content["type"] == "files":
        preview = ", ".join(map(str, content["data"][:3]))
        if len(content["data"]) > 3:
            preview += f"... ({len(content['data'])} files total)"
        data.append(["Files", preview])
    elif content["type"] == "image":
        data.append(["Image Path", content["data"]])
    
    return tabulate(data, headers=["Property", "Value"], tablefmt="grid")

def main():
    """Main function to handle command-line interface."""
    import argparse
    
    parser = argparse.ArgumentParser(description="Windows Clipboard Manager")
    parser.add_argument("--monitor", action="store_true",
                      help="Monitor clipboard for changes")
    parser.add_argument("--get", action="store_true",
                      help="Get current clipboard content")
    parser.add_argument("--search",
                      help="Search clipboard history")
    parser.add_argument("--clear", action="store_true",
                      help="Clear clipboard history")
    parser.add_argument("--history-size", type=int, default=100,
                      help="Maximum number of items in history")
    parser.add_argument("--format", choices=["table", "json"], default="table",
                      help="Output format (table or json)")
    
    args = parser.parse_args()
    
    clipboard_manager = ClipboardManager(args.history_size)
    
    try:
        if args.monitor:
            clipboard_manager.monitor_clipboard()
            
        elif args.get:
            content = clipboard_manager.get_clipboard_content()
            print(format_clipboard_data(content, args.format))
            
        elif args.search:
            results = clipboard_manager.search_history(args.search)
            if args.format == "json":
                print(json.dumps(results, indent=2))
            else:
                if not results:
                    print("No matching entries found")
                else:
                    for content in results:
                        print(format_clipboard_data(content, args.format))
                        print()
                        
        elif args.clear:
            clipboard_manager.clear_history()
            print("Clipboard history cleared")
            
        else:
            parser.print_help()
            
    except Exception as e:
        logger.error(f"An error occurred: {e}")
        sys.exit(1)

if __name__ == "__main__":
    main()
